﻿import os

import app.util
import app.servicecontrol
import app.batch
import app.defaults
import app.logutil

class BootstrapServiceAction(app.servicecontrol.ControllerAction):
    def __init__(self, services, logger):
        self.__services = services
        return super(BootstrapServiceAction, self).__init__(logger)
    
    def chain(self, action):
        assert isinstance(action, BootstrapServiceAction)

        return BootstrapServiceAction(
            self.__services + action.getInvolvedServices(),
            self.logger)
    
    def getInvolvedServices(self):
        return self.__services
    
    def confirm(self, isRedeployment=False):
        if not len(self.__services):
            # Empty action - there is nothing to confirm.
            return True

        self.printLog(
            "warn", 
            "The following services will be bootstrapped: \n  %s\nAny existing data will be lost." % (
                    "\n  ".join(sorted([service.getName() for service in self.__services]))))

        return "y" == app.util.readFromUser("Do you wish to continue? [y/N]:", "y", ["y","N"])
    
    def execute(self):
        allOk = True

        for service in self.__services:
            # A separate batch is run for each service so that we the order
            # corresponds to the order of the services list
            
            scriptsFolder = os.path.join(
                app.defaults.DEFAULT_SERVICE_DIR, 
                service.getName(),
                "hooks",
                "bootstrap")

            if os.path.isdir(scriptsFolder):
                self.printLog('title', "Bootstrapping %s..." % service.getName())

                batch = app.batch.BootstrapBatch(
                    self.logger, 
                    app.util.getRuntimeConfiguration())

                batch.addFolder(scriptsFolder)

                try:
                    batch.execute()
                except Exception, err:
                    self.logger.error(''.join(app.util.getStackTraceAndException()).strip())
                    allOk = False
            
        if not allOk:
            self.printLog("error", "\nBootstrapping failed. See %s for details." % (app.logutil.getFileName(self.logger) or "log"))
 
        return allOk