﻿import app.util
import app.logutil
import platform
import commands

class LinuxService:
    def __init__(self, serviceName,log=None):
        self.serviceName = serviceName
        self.systemctlPath = '/bin/systemctl'
        self.initdPath = '/etc/init.d/'
        self.chkconfigPath = '/sbin/chkconfig'

        if not log:
          self.log = app.logutil.getRuntimeLogger(app.defaults.DEFAULT_LOG_DIR, False)
        else:
          self.log = log
        
        (distname,version,id) = platform.linux_distribution()
        if distname in ['Red Hat Enterprise Linux Server', \
                        'CentOS Linux', 'CentOS']:
           self.dist = 'redhat'
        try:
          self.os_version = int(platform.linux_distribution()[1].split('.')[0])
        except:
          raise Exception("You are running the installation on an unsupported operating system.")
        
    def stopAndDisable(self):
        self.stop()
        self.disable()

    def disable(self):
        if self.os_version >= 7:
          app.util.runSystem(self.systemctlPath + " disable " + self.serviceName \
               + " &> /dev/null" , self.log,skip_error=1)
        elif self.os_version == 6:
          app.util.runSystem(self.chkconfigPath + " " + self.serviceName \
               + " off &> /dev/null", self.log,skip_error=1)
          app.util.runSystem(self.chkconfigPath + " --del " + self.serviceName \
               + " &> /dev/null", self.log,skip_error=1)

    def stop(self):
        if self.os_version >= 7:
          app.util.runSystem(self.systemctlPath + " stop " + self.serviceName \
               + " &> /dev/null"  , self.log,skip_error=1)
        elif self.os_version == 6:
         app.util.runSystem(self.initdPath + self.serviceName \
               + " stop &> /dev/null", self.log,skip_error=1)

    def isEnabled(self):
        exitCode = 0
        if self.os_version >= 7:
          (exitCode, output) = commands.getstatusoutput(self.systemctlPath + " is-enabled "+ self.serviceName \
               + " &> /dev/null" )
        elif self.os_version == 6:
          (exitCode, output) = commands.getstatusoutput(self.chkconfigPath + " " + self.serviceName \
               + " status &> /dev/null" )
        return (exitCode == 0)

    def isRunning(self):
        exitCode = 0
        if self.os_version >= 7:
          (exitCode, output) = commands.getstatusoutput(self.systemctlPath + " status "+ self.serviceName \
               + " &> /dev/null" )
        elif self.os_version == 6:
          (exitCode, output) = commands.getstatusoutput(self.initdPath + self.serviceName \
               + " status &> /dev/null" )
        return (exitCode == 0)

def main():
    for service in ['sendmail', 'postfix', 'httpd']:
      ls = LinuxService(service)
      print ls.serviceName + " " , ls.isRunning()

if __name__ == '__main__':
    main()
