import socket
import sys
import time
import fcntl

g_recursiveAcquisitionCount = 0

class GlobalLock(object):
    def __init__(self):
        self.__socket = None

    def __enter__(self):
        """ 
            Acquire a system-wide lock for this process. It will be held until the 
            process terminates or the 'with' block ends. 
        """
        global g_recursiveAcquisitionCount

        if g_recursiveAcquisitionCount > 0:
            # Lock acquired already
            g_recursiveAcquisitionCount += 1

        else:
            # Without holding a reference to our socket somewhere it gets garbage
            # collected when the function exits
            self.__socket = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
        
            # Do not inherit this socket to child processes
            flags = fcntl.fcntl(self.__socket, fcntl.F_GETFD)
            fcntl.fcntl(self.__socket, fcntl.F_SETFD, flags | fcntl.FD_CLOEXEC)

            try:
                self.__socket.bind('\0TeamForgeRuntime')
            
                # We bound successfully, that means we own the lock now.
                g_recursiveAcquisitionCount += 1

            except socket.error:
                raise LockOwnedByOtherProcess()

    def __exit__(self, type, value, traceback):
        global g_recursiveAcquisitionCount

        g_recursiveAcquisitionCount -= 1
        if self.__socket and g_recursiveAcquisitionCount == 0:
            self.__socket.close()

class LockOwnedByOtherProcess(Exception):
    pass