__copyright__ = "Copyright (c) 2007 CollabNet, Inc. All rights reserved."

from app.scripts import BaseCreateRuntimeScript
import app.util
import app.batch
import service.util
import os
import commands
import sys
import time

try:
   import hashlib as md5
except:
   import md5

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Check and Install CodeSearch application"

    def createRuntime(self):
        propFile = os.path.join(self.cfg['RUNTIME_DIR'], 'conf', 'codesearch.properties')
        codesearchVersion = self.cfg['BDCS_VERSION']
        cmdStr = "rpm -qa --queryformat '%{version}' 'CodeSearch-installer'"
        cmdResult = commands.getstatusoutput(cmdStr);
        if cmdResult[0] != 0:
            self.logMesg("error", "Unable to fetch CodeSearch version details that is installed.")
            sys.exit(1)
        rpmVersion = cmdResult[1]

        installerPath = os.path.join(self.cfg['BDCS_INSTALLER_PATH'], rpmVersion);
        codesearchInstallStatus = self.checkBDCS_InstalledStatus(codesearchVersion, propFile)
        # Return status definition
        # 3 => Supported version Already installed. Skip installation/upgradation/updation
        # 2 => There is change in codesearch config token. Require Update
        # 1 => Unsupported version installed. Requires Upgrade
        # 0 => Codesearch is not installed. Requires fresh Install
        if (codesearchInstallStatus == 3):
           # Every thing is alright. Just start the codesight database service
           self.instructCodeSearchDB('start')
        elif (codesearchInstallStatus == 2):
           self.upgradeBDCS(codesearchVersion, installerPath, propFile)
        elif (codesearchInstallStatus == 1):
           self.upgradeBDCS(codesearchVersion, installerPath, propFile)
        elif (codesearchInstallStatus == 0):
           self.installBDCS(codesearchVersion, installerPath, propFile)

        if service.util.waitForServerUp('localhost', self.cfg['bdcs/bind/jdbc/port'], 30):
           self.updateSysadminAccount();
        self.updateChecksum(propFile);
        return

    # Function to control codesight database service
    def instructCodeSearchDB(self, operation):
        pgsqlCmd  = self.cfg['BDCS_PGSQL_SERVICE_CMD'];
        runCmd = "%s %s 2>&1 > /dev/null"%(pgsqlCmd, operation)
        commands.getstatusoutput(runCmd)

    # Read the md5sum value stored in codesearchInstallStatus.md5 file.
    def readChecksum(self):
       checkSumFile = os.path.join(self.cfg['SITE_DIR'], "etc", "codesearch_properties.md5");
       if (os.path.isfile(checkSumFile)):
          fileFH = open(checkSumFile, 'r')
          checkSum = fileFH.readline().replace("\n","")
          fileFH.close()
       else:
          checkSum = "NULL"
       return checkSum

    # Update the md5sum value in codesearch_properties.md5 file.
    def updateChecksum(self, propFile):
       checkSum = self.getMD5Sum(propFile);
       checkSumFile = os.path.join(self.cfg['SITE_DIR'], "etc", "codesearch_properties.md5");
       fileFH = open(checkSumFile, 'w')
       fileFH.write(checkSum)
       fileFH.close()

    # Get the md5sum checksum for the given file
    def getMD5Sum(self, propFile):
       cmdStr = "md5sum %s  | cut -d ' ' -f1"%propFile
       cmdResult = commands.getstatusoutput(cmdStr);
       if cmdResult[0] != 0:
          self.logMesg("warn", "Unable to get the MD5 checksum of [%s]"%propFile)
       return cmdResult[1]

    # Update Default Sysadmin Account details
    def updateSysadminAccount(self):
        bdcsAdminUser = self.cfg['BDCS_ADMIN_USERNAME']
        bdcsAdminPswd = self.cfg['BDCS_ADMIN_PASSWORD']

        md5Digest = None
        try:
            md5Digest = md5.new()
        except:
            md5Digest = md5.md5()

        md5Digest.update(bdcsAdminPswd)
        bdcsAdminPswdDigest = md5Digest.hexdigest()
        cmdStr = 'su - bds-codesight  -c "psql bds_codesight -c \\"update suite.enduser set name=\'%s\', password=\'%s\' where id=1\\""'%(bdcsAdminUser, bdcsAdminPswdDigest)
        cmdResult = commands.getstatusoutput(cmdStr)
        if cmdResult[0] != 0:
           self.logMesg("error", "Error occurred while updating the Codesearch admin credentials!!.")
           self.logMesg("error", "Reason: [%s]" %cmdResult[1])

    # As of now Both Upgrade and Install follow same steps. This may change in future. So
    # defined separate functions.
    def upgradeBDCS(self, codesearchVersion, installerPath, propFile):
        self.logMesg("debug", "Upgrading CodeSearch Version [%s]."%codesearchVersion)
        self.checkBDCSInstaller_Installed(codesearchVersion)
        installerScript = os.path.join(installerPath, "install.sh");
        cmdStr = "%s -i silent -Dblackduck.installer.properties=%s"%(installerScript, propFile)
        cmdResult = commands.getstatusoutput(cmdStr)
        if cmdResult[0] != 0:
           self.logMesg("error", "Error occurred while installing CodeSearch.")
           self.logMesg("error", "Please see the log file /tmp/BDS_IACA_CodeSight_server.log")
           sys.exit(1)
        self.processLDConfig()
        self.logMesg("debug", "CodeSearch with version [%s] is upgraded successfully."%codesearchVersion)
        return

    # Function for installing Blackduck Codesearch
    def installBDCS(self, codesearchVersion, installerPath, propFile):
        self.logMesg("debug", "Installing CodeSearch Version [%s]."%codesearchVersion)
        self.checkBDCSInstaller_Installed(codesearchVersion)
        installerScript = os.path.join(installerPath, "install.sh");
        cmdStr = "%s -i silent -Dblackduck.installer.properties=%s"%(installerScript, propFile)
        cmdResult = commands.getstatusoutput(cmdStr)
        if cmdResult[0] != 0:
           self.logMesg("error", "Error occurred while installing CodeSearch.")
           self.logMesg("error", "Please see the log file /tmp/BDS_IACA_CodeSight_server.log")
           sys.exit(1)
        self.processLDConfig()
        self.logMesg("debug", "CodeSearch with version [%s] is installed successfully."%codesearchVersion)
        return

    def processLDConfig(self):
        # As per discussion with Blackduck, they said that we can remove this file because it may
        # cause symbol errors in our postgresql deployments.
        fileToRemove = "/etc/ld.so.conf.d/bds-codesight-postgresql.conf"
        if (os.path.isfile(fileToRemove)):
            os.remove(fileToRemove)
        cmdStr = "/sbin/ldconfig 2> /dev/null"
        cmdResult = commands.getstatusoutput(cmdStr)
        return

    # Check the installation status of the Codesearch if already installed.
    # Return status definition
    # 3 => Supported version Already installed. Skip installation/upgradation/updation
    # 2 => There is change in codesearch config token. Require Update
    # 1 => Unsupported version installed. Requires Upgrade
    # 0 => Codesearch is not installed. Requires fresh Install
    def checkBDCS_InstalledStatus(self, codesearchVersion, propFile):
        installAppPath = os.path.join(self.cfg['BDCS_INSTALL_PATH'], 'CodeSight');

        # If Codesearch is not installed. Triggerfresh install
        if not (os.path.isdir(installAppPath)):
           self.logMesg("debug", "CodeSearch is not yet installed. CodeSearch Installer will be triggered now.")
           return 0

        installStatusFile = os.path.join(installAppPath, 'id.txt')
        cmdStr = "sed 's/[a-zA-Z ]//g' %s"%installStatusFile
        cmdResult = commands.getstatusoutput(cmdStr)
        # If Codesearch Installer is not found
        if cmdResult[0] != 0:
           self.logMesg("error", "Unable to fetch CodeSearch version details that is installed.")
           self.logMesg("error", "Reason: Unable to open/process file [%s]"%installStatusFile)
           sys.exit(1)

        installedVersion = cmdResult[1]
        # Require only major.minor version data to check
        installedVersion = ".".join(installedVersion.split(".")[0:2])
        # If Unknown/Unsupported version of codesearch installed, then trigger upgrade/reinstall
        if installedVersion != codesearchVersion:
           self.logMesg("warn", "CodeSearch version installed is [%s]."%installedVersion)
           self.logMesg("warn", "Supported CodeSearch version is [%s]."%codesearchVersion)
           self.logMesg("warn", "Codesearch version [%s] will be installed now."%codesearchVersion)
           return 1

        checkSum = self.getMD5Sum(propFile)
        checkSumPrev = self.readChecksum()
        # If Codesearch properties are change in siteoption,s then trigger update
        if checkSum != checkSumPrev:
           self.logMesg("debug", "Detected codesearch configuration tokens change.")
           self.logMesg("debug", "Will trigger a codesearch updation.")
           return 2

        # Proper Codesearch version is installed. No further action required.
        self.logMesg("debug", "CodeSearch version installed is [%s]. It is supported by Teamforge."%installedVersion)
        self.logMesg("debug", "Skipping CodeSearch installation. CodeSearch already installed.")
        return 3

    # To check if the Codesearch Installer is installed.
    def checkBDCSInstaller_Installed(self, codesearchVersion):
        codesearchPackage = "CodeSearch-installer-%s"%codesearchVersion
        cmdStr = "rpm -qa | grep %s"%codesearchPackage
        cmdResult = commands.getstatusoutput(cmdStr)
        if cmdResult[0] != 0:
           self.logMesg("error", "CodeSearch Installer RPM [%s] is not installed."%codesearchPackage)
           self.logMesg("error", "Please install CodeSearch package [%s] first."%codesearchPackage)
           sys.exit(1)
        return

