__copyright__ = "Copyright (c) 2007 CollabNet, Inc. All rights reserved."

from app.scripts import BaseCreateRuntimeScript
import app.util
import app.batch
import os
import commands
import sys

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Configure CodeSearch tomcat"

    def createRuntime(self):
        tomcatConfPath = os.path.join(self.cfg['BDCS_INSTALL_PATH'], 'CodeSight',
                            'tomcat', 'conf')
        tomcatWebAppConfPath = os.path.join(tomcatConfPath, 'Catalina', 'localhost')
        serverXmlPath = os.path.join(tomcatConfPath, 'server.xml')
        rootXmlPath = os.path.join(tomcatWebAppConfPath, 'ROOT.xml')
        solrXmlPath = os.path.join(tomcatWebAppConfPath, 'solr.xml')

        self.instructCodeSearchTomcat("stop")
        bdcsSSL = self.cfg['BDCS_SSL']
        if bdcsSSL == 'on':
           if self.cfg.has_key('BDCS_SSL_CERT_FILE'):
              self.createTomcat_KeyStore_External()
           else:
              self.createTomcat_KeyStore_Self()
           self.configureTomcat("codesearch-tomcat-ssl.d", serverXmlPath)
        else:
           self.configureTomcat("codesearch-tomcat.d", serverXmlPath)

        self.configureTomcat("codesearch-tomcat-ROOT.xml.d", rootXmlPath)
        self.configureTomcat("codesearch-tomcat-solr.xml.d", solrXmlPath)
        self.updateChecksum(serverXmlPath)
        return

    def instructCodeSearchTomcat(self, operation):
        tomcatCmd = os.path.join(self.cfg['RUNTIME_DIR'], 'scripts', 'collabnet')
        runCmd = "%s %s tomcatcs 2>&1 > /dev/null"%(tomcatCmd, operation)
        commands.getstatusoutput(runCmd)

        tomcatCmd = self.cfg['BDCS_TOMCAT_SERVICE_CMD'];
        runCmd = "%s %s 2>&1 > /dev/null"%(tomcatCmd, operation)
        commands.getstatusoutput(runCmd)

        pgsqlCmd  = self.cfg['BDCS_PGSQL_SERVICE_CMD'];
        runCmd = "%s %s 2>&1 > /dev/null"%(pgsqlCmd, operation)
        commands.getstatusoutput(runCmd)

    def createTomcat_KeyStore_External(self):

        siteDir = self.cfg['SITE_DIR']
        bdcsHost = self.cfg['bdcs/bind/http/host']
        bdcsDomain = self.cfg['bdcs/bind/http/host']

        crtFile = self.cfg['BDCS_SSL_CERT_FILE']
        keyFile = None
        caFile  = None
        chainFile = None
        if self.cfg.has_key('BDCS_SSL_KEY_FILE'):
           keyFile = self.cfg['BDCS_SSL_KEY_FILE']
        if self.cfg.has_key('BDCS_SSL_CA_CERT_FILE'):
           caFile = self.cfg['BDCS_SSL_CA_CERT_FILE']
        if self.cfg.has_key('BDCS_SSL_CHAIN_FILE'):
           chainFile = self.cfg['BDCS_SSL_CHAIN_FILE']

        if keyFile == None:
           self.logMesg("error", "BDCS_SSL_KEY_FILE not defined although BDCS_SSL_CERT_FILE is defined as [%s]!"%(certFile))
           self.logMesg("error", "Please define the full path of SSL Key file in token [BDCS_SSL_CERT_FILE]")
           sys.exit(1)
        elif not os.path.isfile(crtFile):
           self.logMesg("error", "BDCS_SSL_CERT_FILE with value [%s] is invalid or not found!!"%(crtFile))
           sys.exit(1)
        elif not os.path.isfile(keyFile):
           self.logMesg("error", "BDCS_SSL_KEY_FILE with value [%s] is invalid or not found!!"%(keyFile))
           sys.exit(1)
        elif caFile != None and not os.path.isfile(caFile):
           self.logMesg("error", "BDCS_SSL_CA_CERT_FILE with value [%s] is invalid or not found!!"%(caFile))
           sys.exit(1)
        elif chainFile != None and not os.path.isfile(chainFile):
           self.logMesg("error", "BDCS_SSL_CHAIN_FILE with value [%s] is invalid or not found!!"%(chainFile))
           sys.exit(1)

        self.logMesg("debug", "Creating Codesearch keystore using certificate [%s], key [%s]"%(crtFile, keyFile))
        keyToolPath  = os.path.join(self.cfg['JAVA_HOME'], 'jre', 'bin', 'keytool')
        keyStoreFile = os.path.join(siteDir, 'etc', 'codesearch_tomcat.store')

        if os.path.isfile(keyStoreFile):
           os.unlink(keyStoreFile)

        if caFile != None and chainFile != None:
            runCmd = '/usr/bin/openssl pkcs12 -export -chain -CAfile %s -in %s -inkey %s -certfile %s -out %s -name tomcat -password pass:"C0ll@bN3t"'%(chainFile, crtFile, keyFile, caFile, keyStoreFile)
        elif caFile != None and chainFile == None:
            runCmd = '/usr/bin/openssl pkcs12 -export -in %s -inkey %s -certfile %s -out %s -name tomcat -password pass:"C0ll@bN3t"'%(crtFile, keyFile, caFile, keyStoreFile)
        else:
            runCmd = '/usr/bin/openssl pkcs12 -export -in %s -inkey %s -out %s -name tomcat -password pass:"C0ll@bN3t"'%(crtFile, keyFile, keyStoreFile)

        ret = commands.getstatusoutput(runCmd)
        if ret[0] != 0:
           self.logMesg("error", "Unable to generate keystore for Codesearch Tomcat SSL!!")
           self.logMesg("error", "Reason: %s"%(ret[1]))
           sys.exit(1)

        if not self.validateKeyStoreFile(bdcsDomain, keyStoreFile):
           self.logMesg("warn", "The Common name in Codesearch Certificate Key file [%s] doesnot match with your domain name [%s]."%(keyFile, bdcsDomain))
           self.logMesg("warn", "This may make your Teamforge Server unable to communicate with Codesearch Server!!")
        return

    def createTomcat_KeyStore_Self(self):

        siteDir = self.cfg['SITE_DIR']
        bdcsHost = self.cfg['bdcs/bind/http/host']
        bdcsDomain = self.cfg['bdcs/bind/http/host']

        self.logMesg("debug", "Generating Self signed SSL Certificate for Codesearch Keystore....")
        keyToolPath  = os.path.join(self.cfg['JAVA_HOME'], 'jre', 'bin', 'keytool')
        keyStoreFile = os.path.join(siteDir, 'etc', 'codesearch_tomcat.store')

        if os.path.isfile(keyStoreFile):
           os.unlink(keyStoreFile)

        runCmd = 'echo "\n\n" | %s -genkey -alias tomcat -keyalg RSA -storepass "C0ll@bN3t" -keystore %s -storetype "pkcs12" -dname "CN=%s"'%(keyToolPath, keyStoreFile, bdcsDomain)
        ret = commands.getstatusoutput(runCmd)
        if ret[0] != 0:
           self.logMesg("error", "Unable to generate keystore for Codesearch Tomcat SSL!!")
           self.logMesg("error", "Reason: %s"%(ret[1]))
           sys.exit(1)

    def validateKeyStoreFile(self, bdcsDomain, keyStoreFile):

        if not os.path.isfile(keyStoreFile):
           return False
        elif os.path.getsize(keyStoreFile) == 0:
           return False

        keyToolPath  = os.path.join(self.cfg['JAVA_HOME'], 'jre', 'bin', 'keytool')
        runCmd = '%s -list -v -keystore %s -storetype "pkcs12" -storepass "C0ll@bN3t" | grep -i "owner" | sed -e "s/.*CN=//g" -e "s/,.*//g"'%(keyToolPath, keyStoreFile)
        ret = commands.getstatusoutput(runCmd)
        if ret[0] != 0:
           return False

        # Check if the Common Name in Certificate matches with Codesearch Host Name
        certHost = ret[1].rstrip()
        if certHost.find(bdcsDomain) < 0:
           return False

        return True

    def configureTomcat(self, snippetDir, propFile):
        app.batch.ConfigBatch(self.log) \
            .addFolder(os.path.join(app.defaults.DEFAULT_SERVICE_DIR, 'bdcs', 'conf-snippets', snippetDir)) \
            .evaluateIntoFile(propFile)

    # Read the md5sum value stored in codesearch_tomcat_properties.md5 file.
    def readChecksum(self):
       checkSumFile = os.path.join(self.cfg['SITE_DIR'], "etc", "codesearch_tomcat_properties.md5");
       if (os.path.isfile(checkSumFile)):
          fileFH = open(checkSumFile, 'r')
          checkSum = fileFH.readline().replace("\n","")
          fileFH.close()
       else:
          checkSum = "NULL"
       return checkSum

    # Update the md5sum value in codesearch_tomcat_properties.md5 file.
    def updateChecksum(self, propFile):
       checkSum = self.getMD5Sum(propFile);
       checkSumFile = os.path.join(self.cfg['SITE_DIR'], "etc", "codesearch_tomcat_properties.md5");
       fileFH = open(checkSumFile, 'w')
       fileFH.write(checkSum)
       fileFH.close()

    # Get the md5sum checksum for the given file
    def getMD5Sum(self, propFile):
       cmdStr = "md5sum %s  | cut -d ' ' -f1"%propFile
       cmdResult = commands.getstatusoutput(cmdStr);
       if cmdResult[0] != 0:
          self.logMesg("warn", "Unable to get the MD5 checksum of [%s]"%propFile)
       return cmdResult[1]

