#!/bin/bash
# codesightmigration.sh - runs a code sight migration.

# Source the tokens

eval "`$RUNTIME_DIR/scripts/options-to-shell-variables.py`"

usage() {
cat << END_OF_FILE

Usage: `basename $0` [--target=create|status|delete] <target_params>  default: create


Where <target_params> is
    for --target=create:  
            [--filename=<repositories-input-filename>]      default: /tmp/repository-ids
            --username=<admin-username>                  
            --password=<admin-password>		    
            [--totalthreads=<number of threads to spawn>]   default: 25

    for --target=delete:
            [--filename=<repositories-input-filename>]      default: /tmp/repository-ids
            --username=<admin-username>
            --password=<admin-password>
            [--repoids=<repo-id1,repo-id2,repo-id3>]        default : delete all repos as given in input filename
  
    for --target=status:
            [--filename=<repositories-input-filename>]      default: /tmp/repository-ids
            --username=<admin-username>                   
            --password=<admin-password>                  
            [--repoids=<repo-id1,repo-id2,repo-id3>]        default : status for all repos as given in input filename


END_OF_FILE
}

######## configuration values start here #########
#Default target to run
#
TARGET="create"

# Default Input file name
#
INPUT_FILENAME="/tmp/repository-ids"

# Default number of threads to spawn during migration
#
NUMBER_OF_THREADS="25"


RETURNCODE=0
while [ $RETURNCODE -eq 0 ]; do
        case $1 in
        --help|-h)
                usage
                exit
                ;;
        --target*)
                TARGET=`echo $1 | cut -d= -f2`;
                ;;
        --filename*)
                INPUT_FILENAME=`echo $1 | cut -d= -f2`;
                ;;
        --username*)
                ADMIN_USERNAME=`echo $1 | cut -d= -f2`;
                ;;
        --password*)
                ADMIN_PASSWORD=`echo $1 | cut -d= -f2`;
                ;;
        --totalthreads*)
                NUMBER_OF_THREADS=`echo $1 | cut -d= -f2`;
                ;;
         --repoids*)
                REPO_IDS=`echo $1 | cut -d= -f2`;
                ;;
        *)
        esac

        shift
        RETURNCODE=$?
done



if [ ! -n "$JAVA_OPTS" ]; then
	 export JAVA_OPTS="-Xms128m -Xmx1024m -Xss8m"
fi


CODESIGHT_MIGRATE_CLASSPATH="$DISTRIBUTION_DIR/codesight/*:$DISTRIBUTION_DIR/sourceforge_home/sfsoap/lib/*:$DISTRIBUTION_DIR/lib/multiplatform/java/*"

LOG_DIR=$RUNTIME_DIR/../log/codesearch

if [ ! -d "$LOG_DIR" ]; then 
  mkdir $LOG_DIR
fi

LOG_FILE=$LOG_DIR/codesightmigration.log

echo "=========================================================================" >> $LOG_FILE
date >> $LOG_FILE

######## configuration values end here #########

while [  -z "$ADMIN_USERNAME" ]; do
       echo "Please enter valid admin username"
       read ADMIN_USERNAME
done

while [  -z "$ADMIN_PASSWORD" ]; do
       echo "Please enter valid admin password"
       stty -echo
       read ADMIN_PASSWORD	
       stty echo
done

if [ "$TARGET" == 'create' ]; then
	echo "This migration script will create Codesight project for existing Teamforge repositories. At the end of migration each teamforge repository will get mapped to one codesight project.  You must run this script site admin permissions."
elif [ "$TARGET" == 'delete' ]; then
        echo "This migration script will delete existing projects created in codesight server. You must run this script with site admin permissions."
elif [ "$TARGET" == 'status' ]; then 
        echo "This migration script will return status of repositories from codesight server. You must run this script with site admin permissions."
else 
        echo "Please run with correct target."
        exit
fi

while true 
do
	echo -n "Do you want to proceed (y or n) :"
	read CONFIRM
	case $CONFIRM in
	y|Y|YES|yes|Yes) break ;;
	n|N|no|NO|No)
	echo Aborting - you entered $CONFIRM
	exit
	;;
	*) echo Please enter only y or n
	esac
done

echo You entered $CONFIRM. Continuing ...

if [ "$TARGET" == 'create' ]; then
	echo "Executing: $JAVA_HOME/bin/java -cp $CODESIGHT_MIGRATE_CLASSPATH \
	com.collabnet.ce.migration.CodeSightMigration $TARGET $INPUT_FILENAME  $ADMIN_USERNAME  ***** $LOG_DIR $NUMBER_OF_THREADS"
	# Execute migration with correct parameters
	$JAVA_HOME/bin/java -cp "$CODESIGHT_MIGRATE_CLASSPATH" \
	com.collabnet.ce.migration.CodeSightMigration $TARGET "$INPUT_FILENAME"   "$ADMIN_USERNAME"  "$ADMIN_PASSWORD"  "$LOG_DIR" "$NUMBER_OF_THREADS" 2>&1 | tee -a $LOG_FILE 
else 
        echo "Executing: $JAVA_HOME/bin/java -cp $CODESIGHT_MIGRATE_CLASSPATH \
        com.collabnet.ce.migration.CodeSightMigration $TARGET $INPUT_FILENAME  $ADMIN_USERNAME  ***** $LOG_DIR $REPO_IDS"
        # Execute migration with correct parameters
        $JAVA_HOME/bin/java -cp "$CODESIGHT_MIGRATE_CLASSPATH" \
        com.collabnet.ce.migration.CodeSightMigration $TARGET "$INPUT_FILENAME"   "$ADMIN_USERNAME"  "$ADMIN_PASSWORD"  "$LOG_DIR"  "$REPO_IDS" 2>&1 | tee -a $LOG_FILE 
fi       
echo "=========================================================================" >> $LOG_FILE


