#!/bin/bash

# Source the tokens
eval "`$RUNTIME_DIR/scripts/options-to-shell-variables.py`"

export JAVA_HOME
USERNAME="scmviewer"
LOG_FILE=$BDCS_LOG_DIR/post_install.log

# Function that provides APP Url.
get_APPUrl() {
   APP_URL=`echo $1 | awk '{ sub(/sf$/,"",$1); print $1 }' 2> /dev/null`;ret=$?

   if [ "$ret" -ne "0" ] || [ -z $APP_URL ]; then
      debug_Print "Unable to detect APP URL."
      return 1;
   else
      return 0;
   fi
}

# Funtion that provides SCMViewer user password
get_SCMPass() {
   SCM_PASS=`$JAVA_HOME/bin/java -cp $UTIL_JAR_PATH com.vasoftware.sf.system.utils.ScmIntegrationAccessKeyProcessor -decrypt $1`; ret=$?
   if [ $ret -ge 1 ]; then
      debug_Print "Unable to detect SCM Viewer Password."
      debug_Print "Reason: $SCM_PASS"
      return 1;
   else
      return 0;
   fi
}

# Function that invokes a mock svn info request to generate svn auth.simple cache
set_SVNCache() {
    output=`expect << EXPEOF
    set timeout 20
    proc abort {} {
       puts "ERROR: Invalid SCM_USER_ENCRYPTED_PASSWORD or Command Timeout!!\n"
       exit 1
    }
    spawn svn info --username $USERNAME $APP_URL/$SUBVERSION_BRANDING_URI/branding
    expect {
           "Password for" {send $SCM_PASS; send "\r" ; exp_continue}
           "Store password" {send "yes\r" ; exp_continue}
           "Last Changed Date" {send_user "SVNCREDENTIALSSAVED"; close}
           "(R)eject, accept (t)" {send "p\r" ; exp_continue}
           timeout abort
           eof exit
    }
EXPEOF`
    status=`echo $output | grep "SVNCREDENTIALSSAVED" | wc -l`
    if [ $status -eq "1" ]; then
       return 0;
    else
       debug_Print "SVN Transaction is unsuccessful."
       debug_Print "Reason: $output"
       return 1;
    fi
}

debug_Print() {
    echo $1 $2 | tee -a $LOG_FILE
}

debug_Print "Starting Codesearch Post-Install process"
debug_Print "========================================"
debug_Print "`date`"

debug_Print "-en" "Fetching App URL ... "
get_APPUrl $ROOT_URL; ret=$?
if [ $ret -ne "0" ]; then
   debug_Print "ERROR: Error occurred. Exiting !!!"
   exit 1
fi
debug_Print "Done"

debug_Print "-en" "Fetching SCMViewer credentials ... "
get_SCMPass $SCM_USER_ENCRYPTED_PASSWORD; ret=$?
if [ $ret -ne 0 ]; then
   debug_Print "ERROR: Error occurred. Exiting !!!"
   exit 1
fi
debug_Print "Done"

debug_Print "-en" "Generating Subversion cache ... "
set_SVNCache; ret=$?
if [ $ret -ne 0 ]; then
   debug_Print "ERROR: Error occurred. Exiting !!!"
   exit 1
fi

debug_Print "Done"

# calling trust cert
$RUNTIME_SCRIPTS/codesearch/trust-cert.sh

debug_Print "CodeSearch Post Installation process is successful."
debug_Print "========================================"
