#!/usr/bin/env python2

"""
This script reads authorized key from given file and sets the key to scmviewer user profile.
"""
import os
import os.path
import sys
from suds.client import Client
from suds.cache import ObjectCache
import app.util
import getopt
import getpass


def usage():
    print "\nUsage:"
    print "./set_auth_key.py --authkey-file=<path_of_authkey_file>"
    sys.exit(1)

def get_auth_key(file):
    if not os.path.exists(file):
       print "File not found: %s" % file
       sys.exit(1)

    auth_file = open(file, 'r')
    key = "".join(auth_file.readlines())
    return key.replace("\n", "")

def set_auth_key(file):
    key = get_auth_key(file)
    cfg = app.util.getRuntimeConfiguration()
    soap_url = "%s/ce-soap60/services" % cfg.get('API_SOAP')
    sudsCacheDir = os.path.join("%s" %cfg.get('TEMPORARY_DIR'), "suds")
    sudsCache = ObjectCache(sudsCacheDir)
    cnapp = Client("%s/CollabNet?wsdl" %soap_url, cache=sudsCache, location="%s/CollabNet?wsdl" %soap_url)
    username = raw_input("Enter TeamForge username:")
    password = getpass.getpass("password:")
    session_id = cnapp.service.login(username, password)
    scmapp = Client("%s/ScmApp?wsdl" %soap_url, cache=sudsCache, location="%s/ScmApp?wsdl" %soap_url)
    scmapp.service.setAuthorizedKeysForUser(session_id, 'scmviewer', key)
    print "Authorized key is set for scmviewer user"

def main():

    if os.getuid() != 0:
        print "%s: This script requires super user permissions." % os.path.basename(sys.argv[0])
        sys.exit(1)

    try:
        opts, args = getopt.getopt(sys.argv[1:],"h", ["help", "authkey-file="])
    except getopt.GetoptError:
        usage()

    if len(opts) != 1:
        usage()

    auth_file = None
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
        elif opt == '--authkey-file':
            auth_file = arg
        else:
            usage()
            sys.exit()

    if auth_file:
        set_auth_key(auth_file)

if __name__ == "__main__":
    main()
