__copyright__ = "Copyright (c) 2007 CollabNet, Inc. All rights reserved."

from app.scripts import BaseCreateRuntimeScript
import app.util
import app.batch
import service.util
import service.constants
import os
import commands
import re
import time 

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Set up NFS exports"
    
    def createRuntime(self):
        # Should run only if Indexer is not running in App box
        if (service.util.isApplicationEnabledLocally( service.constants.APPLICATION_INDEXER)):
           return

        indexerHost = service.util.getApplicationHost(service.constants.APPLICATION_INDEXER)
        fileStoragePath = self.cfg.get('FILESTORAGE_LOCATION')
        fileStoragePath = os.path.normpath(fileStoragePath)
        exportConfFile  = os.path.normpath("/etc/exports")

        exportEntry = '%s    %s(ro,sync,no_root_squash)'%(fileStoragePath, indexerHost)

        # Adds "filestoragepath   searchhost(ro,sync,no_root_squash)" if not present
        # and restart nfs server daemon
        if (self.checkExportEntry(exportConfFile, exportEntry) == False):
           self.addExportEntry(exportConfFile, exportEntry)
           self.restartNFSDaemon()
           # Hack to restart NFS server twice in RHEL7/CentOS7 for nfs mount to work on indexer box
           # https://forge.collab.net/sf/go/artf200158#11 
           if "redhat-7" in self.cfg["PLATFORM"]:
               time.sleep(5)
               self.restartNFSDaemon()          

    # Function that adds nfs export entry to /etc/exports file
    def addExportEntry(self, confFile, exportEntry):

        aHandler = open(confFile, "a")
        aHandler.write("\n"  + "# Entry for Remote Teamforge Indexer" + "\n")
        aHandler.write(exportEntry + "\n")
        aHandler.close()

    # Function that restarts nfs server daemon
    def restartNFSDaemon(self):
        rpcbindInitCommand = os.path.join("/etc/init.d", "rpcbind")

        if "redhat-7" in self.cfg["PLATFORM"]:
           tfSystemd = os.path.join(self.cfg["RUNTIME_SCRIPTS"], 'tfsystemd')
           nfsInitCommand = "%s nfs" %tfSystemd
           rpcbindInitCommand = "%s rpcbind" %tfSystemd
        else:
           nfsInitCommand = os.path.join("/etc/init.d", "nfs")

        # Restart rpcbind first
        app.util.runSystem("%s restart > /dev/null 2>&1"%rpcbindInitCommand, self.log)
        # Restart nfs daemon. This is to be done only after rpcbind startup.
        app.util.runSystem("%s restart > /dev/null 2>&1"%nfsInitCommand, self.log)

    # Function that checks if the nfs export entry for app filestorage already exists
    def checkExportEntry(self, confFile, exportEntry):

        fHandler = open(confFile, "r")
        exportFileLines = fHandler.read().split("\n")
        fHandler.close()

        # Match the Entry as it begins with the line
        exportEntry = '^' + exportEntry
        exportEntry = exportEntry.replace('/', '\/')
        exportEntry = exportEntry.replace('(', '\(')
        exportEntry = exportEntry.replace(')', '\)')
        matcher = re.compile(exportEntry)
        if (len(filter(matcher.search, exportFileLines)) > 0):
           return True
        return False

