from app.scripts import BaseMigrationScript

import os
import app.util
from app.pgdbadmin import PGCTFDBManager

class MigrationScript(BaseMigrationScript):
    
    scriptDescription = "Migrate CTF Reports Database to latest schema"

    def migrate(self):
       
        jdbcUrl = "jdbc:postgresql://%s:%s/%s" % \
            (self.cfg['ctfcore-datamart-admin/dependency/ctfcore-database/jdbc/host'], self.cfg['ctfcore-datamart-admin/dependency/ctfcore-database/jdbc/port'], self.cfg['DATABASE_NAME'])
                
        reportsJdbcUrl = "jdbc:postgresql://%s:%s/%s" % \
            (self.cfg['ctfcore-datamart-admin/dependency/ctfcore-datamart/jdbc/host'], self.cfg['ctfcore-datamart-admin/dependency/ctfcore-datamart/jdbc/port'], self.cfg['REPORTS_DATABASE_NAME'])
            
        if self.cfg['DATABASE_SSL'] == 'on':
            jdbcUrl = "%s?ssl=true&sslfactory=org.postgresql.ssl.NonValidatingFactory" % jdbcUrl 
            
        if self.cfg['REPORTS_DATABASE_SSL'] == 'on':
            reportsJdbcUrl = "%s?ssl=true&sslfactory=org.postgresql.ssl.NonValidatingFactory" % reportsJdbcUrl         
        
        args = [
            jdbcUrl,
            self.cfg['DATABASE_USERNAME'],
            "".join(["'", self.cfg['DATABASE_PASSWORD'], "'"]),
            "''",
            "true",
            reportsJdbcUrl,
            self.cfg['REPORTS_DATABASE_USERNAME'],
            "".join(["'", self.cfg['REPORTS_DATABASE_PASSWORD'], "'"]),
            "''"
            ]
            
        command = "%s/bin/java -Xms128m -Xmx1024m -Xss8m -jar %s %s" % \
            (self.cfg['JAVA_HOME'], 
             os.path.join(self.cfg['DISTRIBUTION_DIR'], "migratorator/migratorator.jar"), 
             " ".join(args))
       
        self.log.info("About to execute migratorator command: %s" % command)
        self.log.info(app.util.executeCommand(command))
        db = PGCTFDBManager(
            self.cfg.get('REPORTS_DATABASE_NAME'),
            self.cfg.get('REPORTS_DATABASE_USERNAME'),
            self.cfg.get('REPORTS_DATABASE_PASSWORD'),
            self.cfg.get('REPORTS_DATABASE_READ_ONLY_USER'),
            self.cfg.get('REPORTS_DATABASE_READ_ONLY_PASSWORD'),
            self.log,
            self.cfg.get('ctfcore-datamart-admin/dependency/ctfcore-datamart/jdbc/port'),
            self.cfg.get('ctfcore-datamart-admin/dependency/ctfcore-datamart/jdbc/host'),
            self.cfg.get('REPORTS_PGPASSFILE'))

        # Below may not be required for schema changes after 16.10 as
        # bootstrap alter default privileges to accomodate this. Please
        # feel free to remove this when ideal.
        db.grantReadOnlyPermissionsForAllTablesInSchema()
