﻿import os
import app.servicecontrol
import app.migratorator

class ServiceController(app.servicecontrol.ServiceController):
    
    def getDataStatus(self):
        # Deployment is already done, so it is fine to access runtime-options.conf
        config = app.util.getRuntimeConfiguration()
        
        try:

            dbHost = config['ctfcore-datamart-admin-postgres/dependency/ctfcore-database/jdbc/host']
            dbName = config['DATABASE_NAME']
            dbPort = config['ctfcore-datamart-admin-postgres/dependency/ctfcore-database/jdbc/port']
            dbSSL = config['DATABASE_SSL']

            dbUrl = "jdbc:postgresql://%s:%s/%s" % (dbHost, dbPort, dbName)
            if dbSSL == "on":
                dbUrl += '?ssl=true&amp;sslfactory=org.postgresql.ssl.NonValidatingFactory&amp;'


            rdbHost = config['ctfcore-datamart-admin-postgres/dependency/ctfcore-datamart/jdbc/host']
            rdbName = config['REPORTS_DATABASE_NAME']
            rdbPort = config['ctfcore-datamart-admin-postgres/dependency/ctfcore-datamart/jdbc/port']
            rdbSSL = config['REPORTS_DATABASE_SSL']

            rdbUrl = "jdbc:postgresql://%s:%s/%s" % (rdbHost, rdbPort, rdbName)
            if rdbSSL == "on":
                rdbUrl += '?ssl=true&amp;sslfactory=org.postgresql.ssl.NonValidatingFactory&amp;'

   
            if app.migratorator.Migratorator.forDatamart(
                os.path.join(config['DISTRIBUTION_DIR'], "migratorator/migratorator.jar"),
                dbUrl,
                config['DATABASE_USERNAME'],
                config['DATABASE_PASSWORD'],
                rdbUrl,
                config['REPORTS_DATABASE_USERNAME'],
                config['REPORTS_DATABASE_PASSWORD']).isSchemaUpToDate(self.logger):
                
                return app.servicecontrol.ServiceStatus.AVAILABLE
            else:
                return app.servicecontrol.ServiceStatus.NOT_MIGRATED

        except app.migratorator.MigratoratorException:
            if not self.areRequiredEndpointsListening():
                # The database is not even available.
                return app.servicecontrol.ServiceStatus.DEPENDENCIES_UNAVAILABLE 
            else:
                self.logger.warn("Database is available, but invoking migratorator failed - "+
                                 "database probably has not been bootstrapped yet")
                                 
                return app.servicecontrol.ServiceStatus.NOT_BOOTSTRAPPED
                
    def canBootstrap(self):
        if not super(ServiceController, self).canBootstrap():
            return false
            
        # We need Postgres running in order to be able to bootstrap.
        if not self.areRequiredEndpointsListening():
            self.logger.error("Cannot bootstrap as Postgres server is not available")
            return False
        else:
            return True
