__copyright__ = "Copyright (c) 2014 CollabNet, Inc. All rights reserved."


import os
import re
import logging

from app.scripts import BaseInitializationScript
import app.util
import app.logutil
import app.db

class InitializationScript(BaseInitializationScript):

    scriptDescription = "Initialize ETL Job Schedules"

    def __countIncrementalJobRuns(self):
        countIncrementalRunsResult = app.db.runReportingQuery("select count(*) from etl_attributes where attribute_name = 'ALLOW_TRACKER_INCREMENTAL_RUN' and attribute_value = 'false'")
        if type(countIncrementalRunsResult) is list:
            return int(countIncrementalRunsResult[0][0])
        else:
            return 0
    
    def __countFlexFieldJobRuns(self):
        countFlexFieldJobsResult = app.db.runReportingQuery("select count(1) from etl_job where job_name = 'load_flex_fields' and status = 1")
        return int(countFlexFieldJobsResult[0][0])
        
    def __getCurrentSchemaVersion(self):
        schemaVersionResult = app.db.runQuery("select major || '.' || minor || '.' || sp || '.' || hotfix from schema_version")
        return str(schemaVersionResult[0][0])

    def __getSchemaVersionWhenInitialJobRan(self):
        schemaVersionWhenInitialJobRanResult = app.db.runReportingQuery("select ctf_version from etl_job where job_name = 'tracker_initial_etl'")
        return str(schemaVersionWhenInitialJobRanResult[0][0])
    
    def initialize(self):

        logger = app.logutil.getRuntimeLogger()

        incrementalJobCount = self.__countIncrementalJobRuns()
        flexFieldJobCount = self.__countFlexFieldJobRuns()
        
        etlJobs = []
        
        if incrementalJobCount != 0 :
            etlJobs = ['TrackerInitialJob','SCMCommitInitialJob']
            
        elif incrementalJobCount == 0 and flexFieldJobCount == 0:
            if self.__getSchemaVersionWhenInitialJobRan() != self.__getCurrentSchemaVersion():
                etlJobs = ["LoadFlexFields"]

        etlClientScript = os.path.join(self.cfg.get('RUNTIME_SCRIPTS'), "etl-client.py")
           
        for etlJob in etlJobs:
            logger.info("Running ETL job '%s'" % etlJob)
            etlJobCmd = "%s -r %s"%(etlClientScript, etlJob)
            app.util.runCommand(etlJobCmd, logger)
  
