__copyright__ = "Copyright (c) 2014 CollabNet, Inc. All rights reserved."

from app.scripts import BaseInitializationScript
import app.util
import app.landscape
import os
import commands
import shutil

class InitializationScript(BaseInitializationScript):

    scriptDescription = "Import SSL certificates of integration and BDCS servers"

    def __trustHost(self, host, port):
        return app.util.executeCommand(" ".join([
            os.path.join(self.cfg.get('RUNTIME_SCRIPTS'), "trust-cert.sh"),
            host,
            str(port)
            ])) == 0
            
    def __trustScmServers(self):
        if (self.cfg.get('SSL', 'off') != 'on'):
            self.logMesg("debug", "SSL not enabled, importing SSL certificates is not necessary");
            return
            
        for server in app.landscape.Landscape.getSite().getMachinesByService("integration"):
            endpoint = server.getService("integration").getProvidedEndpoint("http-soap", "public")
            
            self.log.info("Importing SSL certificate from %s:%d" % (endpoint.getHost(), endpoint.getPort()))
            self.__trustHost(endpoint.getHost(), endpoint.getPort())
        
            
    def __trustBdcsServers(self):
        if (self.cfg.get('BDCS_SSL', 'off') != 'on'):
            self.logMesg("debug", "BDCS_SSL not enabled, importing SSL certificates is not necessary");
            return
            
        for server in app.landscape.Landscape.getSite().getMachinesByService("bdcs"):
            endpoint = server.getService("bdcs").getProvidedEndpoint("http")
            
            self.log.info("Importing SSL certificate from %s:%d" % (endpoint.getHost(), endpoint.getPort()))
            self.__trustHost(endpoint.getHost(), endpoint.getPort())
    
    def initialize(self):
        self.__trustScmServers()
        self.__trustBdcsServers()
