__copyright__ = "Copyright (c) 2014 CollabNet, Inc. All rights reserved."

from app.scripts import BasePostInstallScript
import app.util
from app.selinux_permissions import SELinuxSetting
import os
import commands,glob
import shutil
import ConfigParser

class PostInstallScript(BasePostInstallScript):

    scriptDescription = "Teamforge CLI Reports Setup"

    cliReportsRPM = "ctf_clireports"
    cliServerRPM  = "CN-cliserver";

    def canRun(self):
        try:
           if self.cfg['TEST_SUPPORT_ENABLED'] == 'true':
              devMode = True
           else:
              devMode = False
        except:
           devMode = False

        if (devMode == False and self.checkPkgInstalled(self.cliReportsRPM) != 0):
           self.logMesg("debug", "Teamforge CLI Reports is not installed. So skipping CLI Reports post installation.")
           self.setSkipMsg("Package %s is not installed"%self.cliReportsRPM)
           return False

        reportsDistDir = os.path.join(self.cfg['DISTRIBUTION_DIR'], "clireports")
        if (devMode == True and not os.path.exists(reportsDistDir)):
           self.logMesg("debug", "Teamforge CLI Reports is not installed. So skipping CLI Reports post installation.")
           self.setSkipMsg("CLIReports Path %s not found"%reportsDistDir)
           return False

        cliServerPKG  = os.path.join(self.cfg['SITE_DIR'], "etc", "add-ons", "CN-cliserver.pkg")
        if (not os.path.isfile(cliServerPKG)):
           self.logMesg("debug", "Teamforge CLI Server package is not installed/configured. So skipping CLI Reports post installation.")
           self.setSkipMsg("Package %s is not installed"%self.cliServerRPM)
           return False

        return True

    def postInstall(self):
        """ Handles CLI Reports Post Installation
               Check if CLI Reports & Server package is installed and configured. If not skip
        """

        reportsDistDir = os.path.join(self.cfg['DISTRIBUTION_DIR'], "clireports")
        reportsDeployDir = os.path.join(self.cfg['DATA_DIR'], "cliserver", "app")
        fcgiDeployDir = "/var/www-local/fcgi-bin"
        modfcgiDir = "/var/run/mod_fcgid"
        arch =  commands.getoutput("uname -m")
        if ( arch != "x86_64" ):
            arch = "i386"
        appUser    = self.cfg["APP_USER"]
        appGroup   = self.cfg["APP_GROUP"]
        httpdUser  = self.cfg["HTTPD_USER"]
        httpdGroup = self.cfg["HTTPD_GROUP"]
        lifeCycleMetricsEnabled = self.cfg["REPORTS_ENABLE_LIFECYCLE_METRICS"]

        clireportsLog  = os.path.join(self.cfg["LOG_DIR"], "cliServer", "cli-reports.log")

        if not os.path.isfile(clireportsLog):
            self.runCmd("touch %s" %clireportsLog, "Create log file for clireports" )
        cmd = "chown %s:%s %s" %(appUser, appGroup, clireportsLog)
        self.runCmd(cmd, "Configure ownership permission to CLI reports log file")
        if ( os.path.isdir("%s/reports" %reportsDeployDir)):
            shutil.rmtree("%s/reports" %reportsDeployDir)
        cmd = "cp -rf %s/reports %s/reports" %(reportsDistDir, reportsDeployDir)
        self.runCmd(cmd, "CLI Reports deploymemt to TeamForge data directory")
        if (lifeCycleMetricsEnabled == 'true'):
           self.setCLIReportsReadOnlyMode('false')
        else:
           self.setCLIReportsReadOnlyMode('true')

        cmd = "rsync -r %s/cron %s" %(reportsDistDir, reportsDeployDir)
        self.runCmd(cmd, "CLI Reports deploymemt to TeamForge data directory")
        shutil.copy("%s/bin/%s/clireports.fcgi" %(reportsDistDir, arch), fcgiDeployDir)
        shutil.copy("%s/events/cli-reports.jar" %reportsDistDir , os.path.join(self.cfg['SITE_DIR'],'var','events'))

        cmd = "chown %s:%s %s/clireports.fcgi" %(appUser, appGroup, fcgiDeployDir)
        self.runCmd(cmd, "Configure ownership permission to CLI reports binaries")
        cmd = "chmod 4755 %s/clireports.fcgi" %fcgiDeployDir
        self.runCmd(cmd, "Configure CLI reports binary file mode")
        cmd = "chown -R %s:%s %s" %(httpdUser, httpdGroup, reportsDeployDir)
        self.runCmd(cmd, "Configure ownership permission to CLI reports binaries")
        cmd = "chown -R %s:%s %s" %(httpdUser, httpdGroup, modfcgiDir)
        self.runCmd(cmd, "Configure ownership permission to mod_fcgi directory")
        self.logMesg("debug", "Deployed Teamforge CLI Reports binaries.")
        self.logMesg("debug", "Restarting apache.")
        self.runCmd("%s restart" %self.cfg['HTTPD_SERVICE_CMD'], "Apache Restart")

        cliSyncScript    = os.path.join(self.cfg['RUNTIME_SCRIPTS'], 'clireports', 'syncreports.py')
        # Set httpd_sys_content_t to syncreports.py for branding post-commit to pass
        if self.cfg['SELINUX_ENABLED'] == "true":
            selinuxsetting = SELinuxSetting()
            selinuxsetting.changeContext('httpd_sys_content_t', cliSyncScript)
            selinuxsetting.changeContext('httpd_log_t', os.path.dirname(clireportsLog))
            if self.cfg.has_key('ORACLE_HOME'):
               oracle_libocci_path = glob.glob(os.path.join(self.cfg['ORACLE_HOME'],"lib","libocci.so.*"))[0]
               selinuxsetting.changeContext('textrel_shlib_t', oracle_libocci_path)
        #Sync Productized CLI Reports to TeamForge DB
        cliReportsVarDir = os.path.join(self.cfg['DATA_DIR'], 'cliserver', 'app', 'reports', 'pkg')
        if (os.path.isdir(cliReportsVarDir) and os.path.isfile(cliSyncScript)):
           cmd = '%s native'%(cliSyncScript)
           self.runCmd(cmd, "Syncing CLI Reports to TeamForge Database")

        #Sync Custom CLI Reports to TeamForge DB
        customCliReportsDir = os.path.join(self.cfg['DATA_DIR'], 'cliserver', 'cli', 'custom-reports', 'pkg')
        if (os.path.isdir(customCliReportsDir) and os.path.isfile(cliSyncScript)):
           cmd = '%s custom'%(cliSyncScript)
           self.runCmd(cmd, "Syncing Custom CLI Reports to TeamForge Database")

        #Setting the httpduser permission for sf-images/cn-reports directory.
        cnReportsImagePath = os.path.join(self.cfg['HTTPD_DOCUMENT_ROOT'],
                                      "sf-images", "cn-reports")
        try:
           cmd = "chown -R %s:%s %s" %(httpdUser, httpdGroup, cnReportsImagePath)
           self.runCmd(cmd, "Configure ownership permission to %s directory"%cnReportsImagePath)
        except Exception, e:
           self.logMesg("debug", "Error while setting the permission for cn-reports image directory.[%s]"%str(e))


        #Add Sync CLI Reports to branding post-commit hook 
        try:
           hookFile = os.path.join(self.cfg['SUBVERSION_BRANDING_REPOSITORY_BASE'], self.cfg['SUBVERSION_BRANDING_REPOSITORY_NAME'], 'hooks','post-commit')
           if (os.path.isfile(hookFile)):
              hookEntry = "%s custom"%(cliSyncScript)
              if self.checkEntryExists(hookFile, hookEntry):
                 # Remove syncreports.py custom entry if already available to
                 # address change issue with change in order of invocation 
                 # of cli_server add-on's update.pl and syncreports.py
                 self.removeEntryFromFile(hookFile, hookEntry)
              self.logMesg("debug", "Adding sync cli reports call to branding repository post-commit hook")
              self.appendEntryToFile(hookFile, hookEntry)
        except:
           self.logMesg("debug", "Error while adding sync cli reports call to branding repository post-commit hook")
           # No need to stop in case of exception as this is not very critical
           pass

        self.logMesg("debug", "Teamforge CLI Reports runtime creation successful.")

        return

    def runCmd(self, cmd, msg=None):

        if msg != None:
           self.logMesg('debug', msg)

        (retVal, outputVal) = commands.getstatusoutput(cmd)
        if (retVal != 0 ):
            self.addError("Error: [%s]"%outputVal)
            raise Exception("RunCommandException")

        self.logMesg('debug', outputVal)
        return retVal

    def checkPkgInstalled(self, packageName):
        rpmCheckCmd  = "rpm -qi %s"%(packageName)
        (retVal, outputVal) = commands.getstatusoutput(rpmCheckCmd)
        return retVal

    def appendEntryToFile(self, inputFile, content):

        wfHandler = open (inputFile, 'a')
        wfHandler.write(content + "\n")
        wfHandler.close()
        return

    def removeEntryFromFile(self, inputFile, entry):
    
        rfHandler = open (inputFile, 'r')
        lines = rfHandler.readlines()
        rfHandler.close()

        #And reopen it in write mode:
        wfHandler = open (inputFile, 'w')
        for line in lines:
            if line!=entry+"\n":
                wfHandler.write(line)
        wfHandler.close()


    def checkEntryExists(self, inputFile, verifyLine):

        fHandler = open(inputFile, 'r')
        fileLines = fHandler.read().split("\n")
        fHandler.close()

        if (len(filter(lambda x: x == verifyLine, fileLines)) > 0):
           return True
        return False

    # Set the CLIReport ReadOnly mode to true or false
    def setCLIReportsReadOnlyMode(self, mode):

        cliConfFile = os.path.join(self.cfg['DATA_DIR'], 'cliserver', 'app', 'reports', 'conf', 'clireports.properties')
        if (not os.path.isfile(cliConfFile)):
           with open(cliConfFile, 'wb') as fH:
              fH.close()
        parseObj = ConfigParser.ConfigParser()
        parseObj.read(cliConfFile);

        if (not parseObj.has_section('admin')):
           parseObj.add_section('admin')
           parseObj.set('admin','clireports','enabled')
           parseObj.set('admin','debug','false')
        parseObj.set('admin','readonlymode', mode)

        with open(cliConfFile, 'wb') as fH:
           parseObj.write(fH)

