__copyright__ = "Copyright (c) 2014 CollabNet, Inc. All rights reserved."

from app.scripts import BasePostInstallScript
import app.util
import service.util
import service.constants
import os
import commands
import app.landscape

class PostInstallScript(BasePostInstallScript):

    scriptDescription = "BlackDuck Codesight Integration"

    bdcsRegistryFile  = "/var/.com.zerog.registry.xml"
    bdcsRPM = "CodeSearch-installer"

    def canRun(self):

        if "redhat-7" in self.cfg["PLATFORM"]:
           self.logMesg("debug", "Codesearch is not supported in platform - %s. So skipping BlackDuck CodeSight post installation."%self.cfg["PLATFORM"])
           self.setSkipMsg("Codesearch is not supported in this platform")
           return False

        if not service.util.isApplicationEnabledLocally(service.constants.APPLICATION_CODESEARCH):
           self.logMesg("debug", "Codesearch is not enabled in HOST token. So skipping BlackDuck CodeSight post installation.")
           self.setSkipMsg("Codesearch is not enabled in HOST token")
           return False

        if (self.checkPkgInstalled(self.bdcsRPM) != 0):
           self.logMesg("debug", "CodeSight Installer %s rpm is not installed. So skipping BlackDuck CodeSight post installation."%self.bdcsRPM)
           self.setSkipMsg("Package %s is not installed"%self.bdcsRPM)
           return False

        if (not os.path.isfile(self.bdcsRegistryFile)):
           self.logMesg("debug", "CodeSight Installer is not installed completely. So skipping BlackDuck CodeSight post installation.")
           self.setSkipMsg("BlackDuck Codesight is not installed"%self.bdcsRPM)
           return False

        return True

    def postInstall(self):

        bdcsPostInstallScript = os.path.join(self.cfg.get('RUNTIME_SCRIPTS'), "codesearch", "post-install.sh")

        self.runCmd(bdcsPostInstallScript, "Running Codesearch post installer script (%s)"%(bdcsPostInstallScript))

        return

    def runCmd(self, cmd, msg=None):

        if msg != None:
           self.logMesg('debug', msg)

        (retVal, outputVal) = commands.getstatusoutput(cmd)
        if (retVal != 0 ):
            self.addError("Error: [%s]"%outputVal)
            raise Exception("RunCommandException")

        self.logMesg('debug', outputVal)
        return retVal

    def checkPkgInstalled(self, packageName):
        rpmCheckCmd  = "rpm -qi %s"%(packageName)
        (retVal, outputVal) = commands.getstatusoutput(rpmCheckCmd)
        return retVal


