__copyright__ = "Copyright (c) 2014 CollabNet, Inc. All rights reserved."

from app.scripts import BasePostInstallScript
import app.util
import service.util
import service.constants
import os
import commands
import time
import subprocess
import select
import app.secureconfig
import app.landscape

class PostInstallScript(BasePostInstallScript):

    scriptDescription = "Gerrit SCM Integration Setup"
    confDataList      = [
                          {
                            'teamforge' : ['tf.siteadmin.username', 'tf.siteadmin.passwd.secure']
                          },
                          {
                            'gerrit'    : ['tf.gerritdb.passwd.secure']
                          }
                        ]
    confDataInfo      = {
                          'tf.siteadmin.username' : 'TeamForge Site-admin Username (Ex: gitadmin/scmadmin)',
                          'tf.siteadmin.passwd.secure' : 'TeamForge Site-admin Password',
                          'tf.gerritdb.passwd.secure' : 'Gerrit Database Password',
                        }

    gerritIntRPM = "ctf-git-integration"

    def canRun(self):

        if not service.util.isApplicationEnabledLocally(service.constants.APPLICATION_GERRIT):
           self.logMesg("debug", "Gerrit is not enabled in HOST token. So skipping gerrit integration.")
           self.setSkipMsg("Gerrit is not enabled in HOST token")
           return False       
 
        if (self.checkPkgInstalled(self.gerritIntRPM) != 0):
           self.logMesg("debug", "CTF Gerrit integration %s rpm is not installed. So skipping gerrit integration."%self.gerritIntRPM)
           self.setSkipMsg("Package %s is not installed"%self.gerritIntRPM)
           return False

        return True

    def postInstall(self):

        gerritPostInstallScript = os.path.join(self.cfg.get('GERRIT_USER_HOME'), "scripts", "post-install.py")
        gerritConfigFile        = "gerrit.config"
        gerritConfigFilePath    = os.path.join(self.cfg.get('GERRIT_USER_HOME'), "etc", gerritConfigFile)
        gerritSecureFile        = "secure.config"
        gerritSecureFilePath    = os.path.join(self.cfg.get('GERRIT_USER_HOME'), "etc", gerritSecureFile)

        secCfg = app.secureconfig.SecureConfig(self.cfg)
        tfUserName = secCfg.getConfig('teamforge', 'tf.siteadmin.username')
        tfUserPswd = secCfg.getConfig('teamforge', 'tf.siteadmin.passwd.secure')
        dbPswd     = secCfg.getConfig('gerrit', 'tf.gerritdb.passwd.secure')

        reconfigOption = ''
        if (self.reconfigure == True):
           reconfigOption = '-r'

        cmdParams = [gerritPostInstallScript, '-u', tfUserName, '-p', tfUserPswd, '-P', dbPswd, reconfigOption]
        self.runCmd(cmdParams, "Running gerrit post installation script (%s)"%(gerritPostInstallScript))

        try:
          import service.runtime_configfiles
          service.runtime_configfiles.setTFMD5Config('gerrit', gerritConfigFile, gerritConfigFilePath)
          service.runtime_configfiles.setTFMD5Config('gerrit', gerritSecureFile, gerritSecureFilePath)
        except:
          pass

        return

    def runCmd(self, cmd, msg=None):

        if msg != None:
           self.logMesg('debug', msg)

        procObj = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        time.sleep(1)

        # Wait for Gerrit integration completion
        while True:
           dispData = self.readProcData(procObj.stdout)
           procObj.stdout.flush()
           if (dispData != ''):
              self.logMesg("debug", dispData)
           if procObj.poll() != None:
              break;

        # Verify Gerrit integration status
        if procObj.returncode == 0:
           self.logMesg("debug", "Integrating Gerrit SCM with Teamforge is successful.")
        else:
           self.addError("Error: While integrating Gerrit scm with Teamforge. Exit code [%s]\n"%procObj.returncode)
           raise Exception("GitServerIntegrationException")

        return procObj.returncode

    def readProcData(self, fObj):

        (r, w, x) = select.select([fObj], [], [], 0.5)
        value = ""
        while r:
            char = fObj.read(1)
            if (len(char) == 0):
               break
            value += char;
            (r, w, x) = select.select([fObj], [], [], 0.5)

        return value;

    def checkPkgInstalled(self, packageName):
        rpmCheckCmd  = "rpm -qi %s"%(packageName)
        (retVal, outputVal) = commands.getstatusoutput(rpmCheckCmd)
        return retVal


