__copyright__ = "Copyright (c) 2014 CollabNet, Inc. All rights reserved."

from app.scripts import BasePostInstallScript
import app.util
import app.db
import app.secureconfig
import service.util
import service.constants

import subprocess
import time
import os.path
import select
import commands
import urllib
from app.pgdbadmin import PGDBAdmin

class PostInstallScript(BasePostInstallScript):

    scriptDescription = "Binary app post install setup"

    confDataList      = [
                          {
                            'teamforge' : ['tf.siteadmin.username', 'tf.siteadmin.passwd.secure']
                          },
                        ]
    confDataInfo      = {
                          'tf.siteadmin.username' : 'TeamForge Site-admin Username (Ex: gitadmin/scmadmin)',
                          'tf.siteadmin.passwd.secure' : 'TeamForge Site-admin Password',
                        }



    def setDatabasePassword(self):
        """ Sets the database password for the given database user """

        db = PGDBAdmin(
            self.cfg.get('IAF_DBNAME'),
            self.cfg.get('IAF_DBUSER'),
            self.cfg.get('IAF_DBPASS'),
            None,
            None,
            self.log,
            self.cfg.get('binary/dependency/binary-database-postgres/jdbc/port'))

        db.setDBPasswd()


    def isApplicable(self):

        if not self.cfg.has_key('BINARY_SETUP_TYPE') or self.cfg.get('BINARY_SETUP_TYPE').lower() == 'none':
           return False

        nodeHost      = service.util.getLocalNodeHostName()
        # If Binary is enabled locally?
        if (not service.util.isApplicationEnabled(service.constants.APPLICATION_BINARY, nodeHost)):
           return False

        (retVal, binaryAppDir) = commands.getstatusoutput("rpm -ql cn-binary | head -3 | tail -1")
        if (len(binaryAppDir) < 1):
           return False
        return True

    def canRun(self):
        return True

    def postInstall(self):

        secCfg = app.secureconfig.SecureConfig(self.cfg)
        tfUserName = secCfg.getConfig('teamforge', 'tf.siteadmin.username')
        tfUserPswd = secCfg.getConfig('teamforge', 'tf.siteadmin.passwd.secure')

        self.setDatabasePassword()
        self.migrateBinaryApp()
        cmd = "cd %s && eval `./bin/env.sh` && ./bin/ctf_configure "%(self.cfg.get('BINARY_APP_DIR'))
        cmd = cmd +  ' -u' + tfUserName + ' -p' + tfUserPswd
        if (self.cfg.get('BINARY_SETUP_TYPE').lower() == 'all'):
           cmd += ' -scn'
        if (self.cfg.get('BINARY_SETUP_TYPE').lower() == 'new'):
           cmd += ' -sn'
        if (self.cfg.get('BINARY_SETUP_TYPE').lower() == 'manual'):
           cmd += ' -s'
        (retVal, outProjAdd) = self.runCmd(cmd, "Running script to add binary integrated application to existing projects ")

        if retVal == 0:
           self.logMesg("debug", "Adding Binary integrated application to existing projects")
        else:
           self.addError("Error: While adding Binary to existing projects. Exitted with code [%s]\n"%retVal)
           #raise Exception("BinaryIntegrationException")
        return


    def runCmd(self, cmd, msg=None):

        if msg != None:
           self.logMesg('debug', msg)
        #self.logMesg("debug", "cmd: %s" % cmd)
        pipe = subprocess.Popen(cmd, shell=True, universal_newlines=True,
                stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        output = str.join("", pipe.stdout.readlines()) 
        returnVal = pipe.wait()
        if returnVal is None:
            returnVal = 0
        self.logMesg("debug", output)
        return returnVal, output

    def migrateBinaryApp(self):

        #cmd = "export PATH=%s/vendor/bundle/ruby/2.1.0/bin:%s/../ruby/bin/:$PATH ; " %(self.cfg['BINARY_APP_DIR'], self.cfg['SITE_DIR'])
        #cmd += "export GEM_HOME=%s/vendor/bundle/ruby/2.1.0/ ; " % ( self.cfg['BINARY_APP_DIR'])
        #cmd += "cd %s && CTF_BASE_URL=%s IAF_DBHOST=localhost " % (self.cfg['BINARY_APP_DIR'], self.cfg['API_SOAP'])
        #cmd += "IAF_DBNAME=%s IAF_DBUSER=%s IAF_DBPASS='%s' " % (self.cfg['IAF_DBNAME'],self.cfg['IAF_DBUSER'],self.cfg['IAF_DBPASS'] )
        #cmd += "RAILS_ENV=production ./bin/rake db:migrate" 
        cmd = "cd %s && eval `./bin/env.sh` && ./bin/rake db:migrate" % self.cfg['BINARY_APP_DIR']
        

        self.logMesg("debug", "rake cmd: %s" %  cmd)
        #import pdb; pdb.set_trace()
        (retVal, migrateOutput) = self.runCmd(cmd)
        if retVal == 0:
           self.logMesg("debug", "Migrate output: %s" %  migrateOutput)
        else:
           self.addError("Error: While migrating Binary App. Exitted with code [%s]\n"%retVal)
