__copyright__ = "Copyright (c) 2007 CollabNet, Inc. All rights reserved."

from app.scripts import BaseCreateRuntimeScript
import app.util
import app.batch
import os
import socket
import commands
import sys
import pwd

class CreateRuntimeScript(BaseCreateRuntimeScript):

    scriptDescription = "Setup PostgreSQL configuration"

    def createRuntime(self):
        # Create postgresql config file
        app.batch.ConfigBatch(self.log) \
            .addFolder(os.path.join(app.defaults.DEFAULT_SERVICE_DIR, 'postgres-datamart', 'conf-snippets', 'reporting-postgresql.conf.d')) \
            .evaluateIntoFile(self.cfg['REPORTS_PGSQL_POSTGRESQL_CONF_FILE_PATH'])

        pgsqlSSL = self.cfg['REPORTS_DATABASE_SSL']
        if pgsqlSSL == "on"   :
            self.handlePgsqlSSL()

    def handlePgsqlSSL(self):
        pgsqlDataDir = self.cfg['REPORTS_PGSQL_DATA_DIR']
        pgsqlSrvCert = os.path.join(pgsqlDataDir, "server.crt")
        pgsqlSrvKey  = os.path.join(pgsqlDataDir, "server.key")

        if not self.validatePgsqlSSLCert(pgsqlSrvCert):
           self.createPgsqlSSLCert(pgsqlSrvCert, pgsqlSrvKey)

    def createPgsqlSSLCert(self, pgsqlSrvCert, pgsqlSrvKey):
        pgsqlDataDir = self.cfg['REPORTS_PGSQL_DATA_DIR']
        myHostname = socket.gethostname();
        pgsqlSrvCsr  = os.path.join(pgsqlDataDir, "server.csr")
        pgsqlSrvPrv  = os.path.join(pgsqlDataDir, "server.pem")

        # Creates Postgresql Certificate signing request file
        certReqCmd = """/usr/bin/openssl req -passout pass:"collabnet" -subj "/C=US/ST=CA/L=Brisbane/O=Teamforge/OU=Teamforge Customer/CN=%s" -keyout %s -new > %s"""%(myHostname, pgsqlSrvPrv, pgsqlSrvCsr)
        self.logMesg("debug", "Creating datamart postgresql server certificate request file for host [%s]"%(myHostname));
        (retVal, outputVal) = commands.getstatusoutput(certReqCmd)
        if (retVal != 0):
           self.logMesg("error", "Error occurred while creating datamart postgresql server certificate request file [%s] for host [%s]"%(pgsqlSrvCsr, myHostname))
           self.logMesg("error", "Reason: %s!!"%(outputVal))
           sys.exit(1)

        # Creates Postgresql Certificate key file from private key
        keyGenCmd = """/usr/bin/openssl rsa -passin pass:"collabnet" -in %s -out %s"""%(pgsqlSrvPrv, pgsqlSrvKey)
        self.logMesg("debug", "Creating datamart postgresql server certificate key file")
        (retVal, outputVal) = commands.getstatusoutput(keyGenCmd)
        if (retVal != 0):
           self.logMesg("error", "Error occurred while creating datamart postgresql server certificate key file [%s]"%(pgsqlSrvKey))
           self.logMesg("error", "Reason: %s!!"%(outputVal))
           sys.exit(1)

        # Creates Selfsigned Postgresql Certificate file from key file
        certGenCmd = """/usr/bin/openssl x509 -in %s -out %s -req -signkey %s -days 36500"""%(pgsqlSrvCsr, pgsqlSrvCert, pgsqlSrvKey)
        self.logMesg("debug", "Creating datamart postgresql server certificate file")
        (retVal, outputVal) = commands.getstatusoutput(certGenCmd)
        if (retVal != 0):
           self.logMesg("error", "Error occurred while creating datamart postgresql server certificate file [%s]"%(pgsqlSrvCert))
           self.logMesg("error", "Reason: %s!!"%(outputVal))
           sys.exit(1)

        # Sets file permission to 0600 and file owners
        postgresUid     = pwd.getpwnam("postgres").pw_uid
        postgresGid     = pwd.getpwnam("postgres").pw_gid
        try:
          for fName in (pgsqlSrvCsr, pgsqlSrvPrv, pgsqlSrvKey, pgsqlSrvCert):
             os.chown(fName, postgresUid, postgresGid)
             os.chmod(fName, 0600)
        except Exception, e:
           self.logMesg("error", "Error occurred while setting permissions and ownership to datamart postgresql SSL certificate files!!!")
           self.logMesg("error", "Reason: %s"%str(e))
           sys.exit(1)


    def validatePgsqlSSLCert(self, pgsqlSrvCert):

        if not os.path.isfile(pgsqlSrvCert):
           return False

        myHostname = socket.gethostname();
        verifyCmd  = """/usr/bin/openssl x509 -in %s -text -noout 2> /dev/null | grep 'Subject:' | sed 's/.*CN=%s.*/true/g;s/.*Subject.*/false/g;'"""%(pgsqlSrvCert, myHostname)
        (retVal, outputVal) = commands.getstatusoutput(verifyCmd)

        if not (retVal == 0 and outputVal == "true"):
           self.logMesg("warn", "Datamart postgresql server certificate file [%s] is not valid for host [%s]."%(pgsqlSrvCert, myHostname))
           return False
        return True

