﻿__copyright__ = "Copyright (c) 2013 CollabNet, Inc. All rights reserved."

from app.scripts import BaseOptionScript

import app.landscape

class OptionScript(BaseOptionScript):
    def optionSnippet(self):
        localServices = app.landscape.Landscape.getSite().getLocalMachine().getServices()
        
        options = []

        # 
        # Provided endpoints.
        #
        for service in localServices:
            serviceName = service.getName()

            # Resolve ambiguity if the same endpoint is available in multiple scopes.
            for endpointId in set([endp.getId() for endp in service.getProvidedEndpoints()]):
                for scope in ["public", "internal", None]:
                    provEndpoint = service.getProvidedEndpoint(endpointId, scope)
                    if not provEndpoint:
                        # Keep looking for other scopes.
                        continue
                        
                    if provEndpoint.isViaOtherPort():
                        # Vias are not useful here, we want the actual port.
                        continue
                        
                if provEndpoint:
                    # Endpoint to bind to. 
                    options.append("%s/bind/%s/host=%s" % (serviceName, endpointId, provEndpoint.getHost(False)))
                    options.append("%s/bind/%s/port=%s" % (serviceName, endpointId, provEndpoint.getPort()))
                                                                                
        # 
        # Required endpoints.
        #
        for service in localServices:
            serviceName = service.getName()

            for reqEndpoint in service.getRequiredEndpoints():
                if reqEndpoint.getProviderEndpoint():
                    options.append("%s/dependency/%s/%s/host=%s" % (
                        serviceName, 
                        reqEndpoint.getService(), 
                        reqEndpoint.getId(), 
                        reqEndpoint.getProviderEndpoint().getHost()))
                    options.append("%s/dependency/%s/%s/port=%s" % (
                        serviceName, 
                        reqEndpoint.getService(), 
                        reqEndpoint.getId(), 
                        reqEndpoint.getProviderEndpoint().getPort()))
                else:
                    # No provider available for this endpoint, must be an endpoint
                    # scoped as 'optional'.
                    pass
                    
        self.log.info("Endpoints: \n- %s" % "\n- ".join(sorted(options)))

        return "\n".join(options)