from app.scripts import BaseMigrationScript

import os
import app.util
from app.pgdbadmin import PGCTFDBManager

class MigrationScript(BaseMigrationScript):
    
    scriptDescription = "Migrate CTF Database to latest schema"
    
    def migrate(self):
     
        jdbcUrl = "jdbc:postgresql://%s:%s/%s" % \
            (self.cfg['ctfcore-database-admin-postgres/dependency/ctfcore-database/jdbc/host'], self.cfg['ctfcore-database-admin-postgres/dependency/ctfcore-database/jdbc/port'], self.cfg['DATABASE_NAME'])
                
        if self.cfg['DATABASE_SSL'] == 'on':
            jdbcUrl = "%s?ssl=true&sslfactory=org.postgresql.ssl.NonValidatingFactory" % jdbcUrl 
                        
        args = [
            jdbcUrl,
            self.cfg['DATABASE_USERNAME'],
            "".join(["'", self.cfg['DATABASE_PASSWORD'], "'"]),
            "''",
            "false"]
            
        command = "%s/bin/java -Xms128m -Xmx1024m -Xss8m -jar %s %s" % \
            (self.cfg['JAVA_HOME'], 
             os.path.join(self.cfg['DISTRIBUTION_DIR'], "migratorator/migratorator.jar"), 
             " ".join(args))

        self.log.info("About to execute migratorator command: %s" % command)
        self.log.info(app.util.executeCommand(command))
        db = PGCTFDBManager(
            self.cfg.get('DATABASE_NAME'),
            self.cfg.get('DATABASE_USERNAME'),
            self.cfg.get('DATABASE_PASSWORD'),
            self.cfg.get('DATABASE_READ_ONLY_USER'),
            self.cfg.get('DATABASE_READ_ONLY_PASSWORD'),
            self.log,
            self.cfg.get('ctfcore-database-admin/dependency/ctfcore-database/jdbc/port'),
            self.cfg.get('ctfcore-database-admin/dependency/ctfcore-database/jdbc/host'),
            self.cfg.get('PGPASSFILE'))

        # Below may not be required for schema changes after 16.10 as 
        # bootstrap alter default privileges to accomodate this. Please 
        # feel free to remove this when ideal.
        db.grantReadOnlyPermissionsForAllTablesInSchema()
        db.revokePermissionForTables(['sfuser', 'password_history'])
